<?php
/**
* User Manager Class for Catviz
*
* Copyright (C) 2003 Joost Horward
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License 
* version 2 as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330,
* Boston, MA 02111-1307 USA
*
*
* This module performs the user & group admin, assigning of rights
* as well as login and logout
*
* @package catviz
* @subpackage userman
* @author Joost Horward <joost@catviz.org>
* $Date: 2004/10/14 21:52:13 $
* $Author: joosth $
* $Log: mod_userman.php,v $
* Revision 1.12  2004/10/14 21:52:13  joosth
* fixed getButton() typo
*
* Revision 1.11  2004/10/12 21:08:20  joosth
* Fixed change password bug - there was no check wether OK or Cancel was clicked.
*
* Revision 1.10  2004/10/06 22:08:52  joosth
* Took forms out of modules and placed them in separate files - reduced module load time by 50%.
*
* Revision 1.9  2004/09/13 21:14:59  joosth
* Changed to GPL License
* Changed password fail message
* Changed email to use site name from registry
*
* Revision 1.8  2003/10/11 19:58:51  joosth
* Changed TDocumentClassSelectorField to use
* use runquery instead of mysql functions
*
* Revision 1.7  2003/07/21 20:41:55  joosth
* Added items
*
* Revision 1.6  2003/06/24 22:25:13  joosth
* Added new fields to mod_users table
* Added account approve/disable dialog with email notification
* Added lost password service
* Added lockout after too many tries (for non admin)
*
* Revision 1.5  2003/05/25 22:14:17  joosth
* Changed to be XHTML 1.0 compliant.
*
* Revision 1.4  2003/05/15 21:45:25  joosth
* Added detail table relationships. Fixed bug in login: rights were not correctly established with empty acl or empty groupmember lists. Fixed bug in hasAccessClass - now works properly for admin even if access_classes array is empty.
*
* Revision 1.3  2003/05/09 22:31:53  joosth
* Changed to use JIT Forms
*
* Revision 1.2  2003/04/29 22:18:18  joosth
* Added TDocumentClassSelectorField
* Added read Public document rights for Everyone
* Added several form rights settings
* Added user change own password (if userman user)
* Added hasDocumentReadClasses function
*
* $Source: /cvsroot/catviz/catviz/mod/userman/php/mod_userman.php,v $
*/

/**
* Document Class selection field
*
* This is a TField descendant which shows a drop-down box
* with possible document classses the user can choose from
* It will only show document classes of which the user has write
* rights. It uses document_class_id as the foreign key name.
*/

class TDocumentClassSelectorField extends TKeySelectorField{
    function TDocumentClassSelectorField(&$theform){
        $this->TKeySelectorField(&$theform,'document_class_id','Document Class','Public','','document_class_id','document_class_name','mod_users_document_class');
    }
    /**
    * Generate the drop-down field
    * @param string $value The value to be shown as default
    * @return string The HTML code for editing this field
    * This is 90% a copy of the TKeySelectorfield method. Maybe the class needs redesigning.
    */
    function on_Edit (){
        $sql="select $this->keyindex as keyindex,$this->keyname as keyname from $this->table ";
        $varname =  "tmodule_" . $this->name;
        $sql_result = runquery($sql);
        while ($row = mysql_fetch_array($sql_result)) {
            $thisindex=$row["keyindex"];
            $thisname=$row["keyname"];
            if ($GLOBALS['module']['mod_userman']->hasDocumentWriteClass($thisindex)){
                $options[$thisindex]=$thisname;
            }
        }

        $content="<select name = \"$varname\">";
        if ($options) {
	        foreach ($options as $optionindex =>$optionname) {
	            $selected = $this->_value==$optionindex ? "selected=\"selected\"" : "";
	            $content .="<option value=\"$optionindex\" $selected>$optionname</option>"; 
	        }
        }
        $content .= "</select>\n";        
                
        return ($content);
    }
}

/**
* User Manager Class
* @package catviz
* @subpackage userman
* @author Joost Horward
* @see TModule
*/

class MOD_userman extends TModule{
    /**
    * This variable holds the access classes that this user owns
    */
    var $access_classes;
    /**
    * Set to true if user is admin
    */
    var $_adminflag;
    /**
    * The username of the currently logged in user
    */
    var $user_name;
    /**
    * The user ID of the currently logged in user
    */
    var $user_id;
    /**
    * The document classes the currently logged in user has read access to
    */
    var $read_document_classes;
    /**
    * The document classes the currently logged in user has write access to
    */
    var $write_document_classes;
    
    var $first_name;
    var $last_name;
    var $login_count;
    var $last_login_date;
    var $last_login_time;

    /**
    * Constructor for the phonebook class
    *
    * @param none
    * @return none
    */

    function MOD_userman(){
        TModule::TModule("userman","User Manager");
        $this->access_classes=array();
        // everyone can read public docs. -> Public needs to have key ID 1
        $this->read_document_classes=array();
        $this->read_document_classes[]=1;
        $this->write_document_classes=array();
    }

    /**
    * Function called after the login form is submitted
    *
    * Determine if userID and password correspond to a user account
    * Get Access Classes and Document Classes this user has access to
    * and store them in the userman object (which is in the session)
    * for quick access
    */    
    function loginfunc(&$form){
        global $module;
        $thebutton=getButton();
        if ($thebutton=="OK") {
        	        
	        $user_name=$form->fields["userid"]->getValue();
	        $password=$form->fields["password"]->getValue();
	        $sql_result = runquery("select password,admin,user_id,last_login_date, last_login_time, first_name, last_name, login_count, user_status,user_approval_date from mod_users where username = \"$user_name\";");
	        if ($row = mysql_fetch_array($sql_result)) {
	            if (($row["password"]==$password) & (($row["user_status"]=="enabled") | ($row["user_status"]=="approved"))) { 
	            	if ($row["user_status"]=="approved"){
	            			$maxdays= $GLOBALS['registry']->getValue('userman','main','login_within_days');
	            		if ((time()-strtotime($row["user_approval_date"]))<3600*24*$maxdays){
	            			runquery("update mod_users set user_status=\"enabled\" where username = \"$user_name\"");
	            			$first_login=TRUE;
	            		} else {
	            			runquery("update mod_users set user_status=\"disabled\" where username=\"$user_name\"");			
	            			$mailmessage=" After waiting too long after account approval the following account has been disabled:\n\n";
	            			$mailmessage.="User ID : $user_name\n\n";
	        				$this->notifyAdmin('Catviz admin alert: user account disabled',$mailmessage);
							return array('userman','too_late','show');            			
	            		}            			
	            	}
	                $this->user_name=$user_name;
	                $this->user_id=$row["user_id"];
	                $this->loggedin=TRUE;
	                $this->_adminflag=$row["admin"];
	                $this->last_login_date=$row["last_login_date"];
	                $this->last_login_time=$row["last_login_time"];
	                $this->first_name=$row["first_name"];
	                $this->last_name=$row["last_name"];
	                $this->login_count=$row["login_count"];
	                
	                $this->access_classes=array();
	                if ($this->_adminflag){
	                    $sql_result=runquery('select document_class_id from mod_users_document_class');
	                    while ($row=mysql_fetch_array($sql_result)) {
	                        $this->read_document_classes[]=$row['document_class_id'];
	                        $this->write_document_classes[]=$row['document_class_id'];
	                    }
	                }else{
	                    $sql1_result=runquery("select access_class_name from mod_users_acl where user_id = $this->user_id;");
	                    while ($row1=mysql_fetch_array($sql1_result)) {
	                        $this->access_classes[]=$row1["access_class_name"];
	                    }
	                    
	                    $sql2_result=runquery("select access_class_name from mod_users_groupmember left join mod_users_group_acl using (group_id) where user_id=$this->user_id");
	                    while ($row2=mysql_fetch_array($sql2_result)) {
	                        $this->access_classes[]=$row2["access_class_name"];
	                    }
	                    
	                    $sql5_result=runquery("select access_class_name from mod_users_group_acl where group_id=1");
	                    while ($row5=mysql_fetch_array($sql5_result)) {
	                        $this->access_classes[]=$row5["access_class_name"];
	                    }                    
	                    
	                    // Get the user document classes
	                    $sql3_result=runquery("select document_class_id ,read_access,write_access from mod_users_user_document_classes where user_id=$this->user_id");
	                    while ($row3=mysql_fetch_array($sql3_result)) {
	                        if ($row3['read_access']){
	                            $this->read_document_classes[]=$row3['document_class_id'];
	                        }
	                        if ($row3['write_access']){
	                            $this->write_document_classes[]=$row3['document_class_id'];
	                        }
	                    }                
	                    $sql4_result=runquery("select document_class_id ,read_access,write_access from mod_users_groupmember left join mod_users_group_document_classes using(group_id) where user_id=$this->user_id or mod_users_groupmember.group_id=1");
	                    while ($row4=mysql_fetch_array($sql4_result)) {
	                        if ($row4['read_access']){
	                            $this->read_document_classes[]=$row4['document_class_id'];
	                        }
	                        if ($row4['write_access']){
	                            $this->write_document_classes[]=$row4['document_class_id'];
	                        }
	                    }
	                }
	
					runquery("update mod_users set failed_login_attempts=0,login_count=login_count+1,last_login_date=CURDATE(), last_login_time=CURTIME() where username = \"$user_name\"");
					
	                // jump to the welcome form
	                $module['mod_menu']->menuHide('login');
	                $module['mod_menu']->menuShow('logout');
	                if ($first_login){
	                	return array('userman','first_login','show');
	                } else{	
	                	return array('userman','welcome','show');
	                }
	            }
	        }
	        runquery("update mod_users set failed_login_attempts=failed_login_attempts+1 where username = \"$user_name\"");
	        $sql_result=runquery("select failed_login_attempts,admin,user_status from mod_users where username = \"$user_name\"");
	        $row=mysql_fetch_array($sql_result);
	        $maxattempts=$GLOBALS['registry']->getValue('userman','main','max_failed_logins');
	        if (($row["failed_login_attempts"]>$maxattempts) & ($row["admin"]!=1) &($row["user_status"]=="enabled"))
	        {
	        	$mailmessage  = "After reaching the maximum number of login attempts the following account has been disabled:\n\n";
	            $mailmessage .= "User ID : $user_name\n";
	        	$this->notifyAdmin('Catviz admin alert: user account disabled',$mailmessage);
	        	
	        	runquery("update mod_users set user_status=\"disabled\" where username=\"$user_name\"");
	        		        	
	        	return array('userman','disabled','show'); 
	        }
	        return array('userman','denied','show');
        }
    }
    
    function forgot_password_func(){
    	$email_address=$this->forms["forgot_password"]->fields["email_address"]->getValue();
    	//echo "The email address is $email_address <br />";
        $sql_result = runquery("select admin, password, user_id, username, first_name, last_name, user_status, email from mod_users where email = \"$email_address\";");        
        $row=mysql_fetch_array($sql_result);
        //echo "The user status is {$row['user_status']} <br />";
        //echo "The admin field is {$row['admin']} <br />";
        
        if (($row['user_status']=='enabled') & ($row['admin']==0)) {
        	$mailmessage  = "This is a message from the forgotten password service.\n";
        	$mailmessage .= "If you have not requested this message please notify the site administrator.\n\n";
        	// generate a password
          	for ($i=1;$i<9;$i++){
    			$r=mt_rand(0,35);    		
    			if ($r<10) {
    				$password .= chr(0x30+$r);
    			}else{
    				$password .= chr(0x41-10+$r);
    			}
    		}        	
    		
    		$mailmessage .= "Your user name is            : {$row['username']}\n";
        	$mailmessage .= "Your password has changed to : $password\n\n";
        	$md5_password=md5($password);
        	//echo "sending email to {$row['email']} <br />";
        	$sitename=$GLOBALS['registry']->getValue('','site_info','name');        	
            mail($row['email'],"Message from $sitename",$mailmessage);	
            runquery("update mod_users set password=\"$md5_password\" where username=\"$user_name\"");
            return array('userman','password_sent','show');        	
        } else{
        	return array('userman','cannot_send_password','show');
        }
    }

    /**
    * submit a changed password
    */
    function submitChangePassword(){
        global $module;
        if (getButton()=='OK'){
	        $old_password=$this->forms["admin"]->fields["old_password"]->getValue();
	        $new_password1=$this->forms["admin"]->fields["new_password1"]->getValue();
	        $new_password2=$this->forms["admin"]->fields["new_password2"]->getValue();
	        $sql_result = runquery("select password,admin,user_id from mod_users where username = \"$this->user_name\";");
	        if ($row = mysql_fetch_array($sql_result)) {
	            if ($row["password"]==$old_password) {
	                if ($new_password1==$new_password2){
	                    $sql_result = runquery("update mod_users set password=\"$new_password1\" where username = \"$this->user_name\";");
	                    if ($sql_result){
	                        return array ('userman','password_success','show');
	                    }
	                }
	            }
	        }
	        return array ('userman','password_fail','show');
        }        
    }    
    
    /**
    * Log out
    *
    * Clear the login info in the userman object
    */
    function On_logout(){
        $this->user_name="";
        $this->user_id="";
        $this->access_classes=array();
        $this->read_document_classes=array();
        $this->read_document_classes[]=1;
        $this->write_document_classes=array();
        $this->_adminflag=FALSE;
        $this->loggedin=FALSE;
        $_SESSION['module']['mod_menu']->menuHide('logout');
        $_SESSION['module']['mod_menu']->menuShow('login');
    }
    
    /**
    * Called to prepare the enable_disable form
    */
    function enable_disable_prepare_func(&$form){
    	//$form=&$theform;
         if ($GLOBALS['req_foreign_key_value']!="") {
         $this->foreign_key_value=$GLOBALS['req_foreign_key_value'];
         }
    	$sql_result=runquery("select username, first_name, last_name,email,login_count,failed_login_attempts,last_login_date,last_login_time,admin,user_status from mod_users where user_id=$this->foreign_key_value");
    	$row=mysql_fetch_assoc($sql_result);
	
		
		$form->fields['username']->setValue($row['username']);
		
		foreach ($row as $fieldname => $fieldvalue) {
			$form->fields[$fieldname]->setValue($fieldvalue);
		}
		
		
		
    	$form->fields['radio_action']->setValue('Set account to disabled');
    	$form->fields['send_email']->setValue(1);
		
    }
    
    function enable_disable_func(&$form){
    	$thebutton=getButton();
        	if ($thebutton=="OK") {
        		switch ($form->fields['radio_action']->getValue()) {
        			case 'Set account to disabled':
        				runquery("update mod_users set user_status=\"disabled\" where user_id=$this->foreign_key_value");
						if ($form->fields['send_email']->getValue()==1)	{												
        				  	mail($form->row['email'],'Message from Catviz',"Your account on the Catviz website has been disabled\n");
        				  	$mailmessage=" ,email sent";        				  	
						}   
						$GLOBALS['content']['statusline']="[OK: account disabled$mailmessage]";     				
        			break;
        			case 'Set account to approved':
        				runquery("update mod_users set user_status=\"approved\", failed_login_attempts=0,user_approval_date=curdate() where user_id=$this->foreign_key_value");
        				if ($form->fields['send_email']->getValue()==1)	{					
        				  	mail($form->row['email'],'Message from Catviz',"Your account on the Catviz website has been approved\n");
        				  	$mailmessage=" ,email sent";
						}
						$GLOBALS['content']['statusline']="[OK: account approved$mailmessage]";
        			break;
        		}

        	}        
    }
    
    function express_account_setup_prepare_func(&$form){
        // Generate password
    	for ($i=1;$i<9;$i++){
    		$r=mt_rand(0,35);    		
    		if ($r<10) {
    			$password .= chr(0x30+$r);
    		}else{
    			$password .= chr(0x41-10+$r);
    		}
    	}
    	$form->fields['password']->setValue(strtolower($password));
    	$form->fields['send_email']->setValue(1);
    	$form->fields['user_expiry_date']->setValue(date('Y-m-d',time()+24*3600*$GLOBALS['registry']->getValue('userman','main','default_expiry_date_days')));
    }
    
   	function express_account_setup_func(&$form){
    	$thebutton=getButton();
        	if ($thebutton=="OK") {
        		/* Check user ID is unique */
        		$username=$form->fields['username']->getValue();
        		$sql_result=runQuery("select count(*) as thecount from mod_users where username=\"$username\"");
        		$row=mysql_fetch_array($sql_result);

        		if ($row['thecount'] >0) {
        			return array('userman','userid_exists','show');
        		}
        		
        		/* Add user to database */
        		
        		$first_name=$form->fields['first_name']->getValue();        		
        		$last_name=$form->fields['last_name']->getValue();
        		$email=$form->fields['email']->getValue();
        		$clear_password=$form->fields['password']->getValue();
        		$password=md5($clear_password);
        		$admin=$form->fields['admin']->getValue();
        		$expiry_date=$form->fields['user_expiry_date']->getValue();
        		$user_creation_date=date('Y-m-d');
        		$query  = "insert mod_users set username=\"$username\", first_name=\"$first_name\", last_name=\"$last_name\", email=\"$email\", password=\"$password\", admin=\"$admin\", user_expiry_date=\"$expiry_date\",";
        		$query .= "user_status=\"approved\",user_creation_date=\"$user_creation_date\", user_approval_date=\"$user_approval_date\", failed_login_attempts=0,login_count=0";        		
        		runquery($query);
  
         		$sql_result=runquery("select last_insert_id() as new_user_id");
        		$row=mysql_fetch_array($sql_result);
        		$user_id=$row['new_user_id'];
        		/* Add groups */
        		$groups=$form->fields['groups']->getValue();

        		//print "Groups: $groups <br />";
        		if ($groups){
	        		$group_array=explode(',',$groups);
	        		foreach ($group_array as $group){
	        			runQuery("insert mod_users_groupmember set user_id=$user_id, group_id=$group");
	        		}
        		}
        		/* Send email if needed */
        		if ($form->fields['send_email']->getValue()==1){
        			$site_name=$GLOBALS['registry']->getValue('','site_info','name');
        			mail($form->fields['email']->getValue(),"New account on $site_name","Dear $first_name $last_name,\n\nAn account has been created for you on $site_name.\n\nUsername: $username\nPassword: $clear_password\n\nThis email was generated by Catviz");
        			$mailmessage=", email sent";
        		}
        		$GLOBALS['content']['statusline']="[OK: account created$mailmessage]";
        		
        	}
    }
    
    /**
    * Return true if the currently logged in user is admin
    * @return boolean True if user is admin
    */
    function isAdmin(){
        return $this->_adminflag;
    }
    /**
    * Return true if the currently logged in user owns the
    * access class indicated
    * @return True if the user has the indicated access class
    */
    function hasAccessClass($access_class){
        if ($this->isAdmin()) {
            return TRUE;
        }
        if ($this->access_classes){
            return in_array($access_class,$this->access_classes);
        }else{
            return FALSE;
        }
    }
    /**
    * Get the list of document classes to which the currently logged in user
    * has read access
    * @return string A comma separated list of document classes
    */
    function getDocumentReadClasses(){
        if ($this->read_document_classes){    
            foreach ($this->read_document_classes as $document_class) {
                $content .= "$document_class,";            
            }
        }
        $content .='1'; // Everyone has public access
        return $content;
    }
    
    /**
    * Return TRUE if this user has read rights to the indicated document class
    * @param int $document_class The document class in question
    * @return boolean True if user has document class read rights
    */    
    function hasDocumentReadClass($document_class){
        return $this->isAdmin() | in_array($document_class,$this->read_document_classes);
    }
    
    /**
    * Get the list of document classes to which the currently logged in user
    * has write access
    * @return string A comma separated list of document classes
    */
    function getDocumentWriteClasses(){
        if ($this->write_document_classes) {    
            foreach ($this->write_document_classes as $document_class) {
                if ($content){
                    $content .= ",$document_class";
                } else{
                    $content .= "$document_class";
                }
            }
            return $content;
        }
    }
    /**
    * Return TRUE if this user has write rights to the indicated document class
    * @param int $document_class The document class in question
    * @return boolean True if user has document class write rights
    */
    function hasDocumentWriteClass($document_class){
        return $this->isAdmin() | in_array($document_class,$this->write_document_classes);
    }
    
    /**
    * Send mail to system admin(s)
    * @param string $mail_subject The subject of the mail to be sent
    * @param string $mail_content The content of the mail to be sent
    */
    
    function notifyAdmin($mail_subject,$mail_content){ 
    	runquery("update mod_users set user_status=\"disabled\" where username=\"$user_name\"");
        $result=runquery("select * from mod_users left join mod_users_acl using (user_id) left join mod_users_groupmember using (user_id) left join mod_users_group_acl using (group_id) where admin=1");            
		while ($row= mysql_fetch_array($result)){
            	mail($row['email'],'Catviz admin alert: user account disabled',$mail_content);
    	}
    }
    
}
?>
